# Notes (Progress)

The intention of this file is to prcedurally log what changes to pm_static.yml have what effects.

## Objective

The objective of this zephyr_experiment is to find a valid configuration such that DFU can be ported to xiao.

## 1. Background

We encountered a flash region overflow while porting to xiao, so we are attempting to make the partitions within primary flash larger so that the app can sit within them.

To that end, we are attempting to move storage_settings into external flash with the goal of expanding mcuboot primary and secondary regions.

## Change notes

From default: (git commit `243fa9f`)

### Change settings_storage to arbitrary name

The name of storage_settings does not affect the ability to compile while it resides in flash_primary under all the same size partitions

During runtime, no errors are thrown, but bonds are not restored after device is reset. 

Return storage_settings to normal

### Add arbitrarily named partition to external flash

* Place the partition immediately following fatfs_storage
    * First attempt - size not specified, pm unhappy for obvious reasons (Key: 'size' not found)
    * Second attempt - specifying size as 0x2000
      * Successful build. Exploring flash behavior.
      * ![alt text](image.png)
      * Strangely, bond does not cause auto-reconnect. We will disregard behavior of bond because it is not validated on this app.

Since the first attempt worked, next step is to try swapping storage and arbitrary partitions

* Swap regions of arbitrary_partition and storage_partition. Validate functionality
  * Builds, no unexpected warnings
  * Operation as expected. DFU process works as expected

Since this worked, we will now attempt to modify the size of the arbitrary partition.

```
==== Before ====
arbitrary_partition:
  address: 0xfe000
  region: flash_primary
  size: 0x2000

==== After ====
arbitrary_partition:
  address: 0xfe000
  region: flash_primary
  size: 0x1000
```

As expected, this causes the "Gaps" error we've been encountering while trying to manipulate the pm_static.yml

Further, the 'extra' gap is in the exact region we expect it. [0xff 000 - 0x100 000] (the last 1000 bytes that we just "shrunk" our partition out of)

```
Partition manager failed: Incorrect amount of gaps found in static configuration. There must be exactly one gap in the static configuration to support placing the dynamic partitions (such as 'app'). Gaps found (2):0xc200-0x85000 0xff000-0x100000 The most common solution to this problem is to fill the smallest of these gaps with statically defined partition(s) until there is only one gap left. Alternatively re-order the already defined static partitions so that only one gap remains.
Failed to partition region flash_primary, size of region: 1048576
Partition Configuration:
app:
  size: 495104
arbitrary_partition:
  size: 4096
mcuboot:
  size: 49152
mcuboot_pad:
  size: 512
mcuboot_primary:
  size: 495616
mcuboot_primary_app:
  size: 495104
mcuboot_secondary:
  size: 495616
```

* Our next hypothesized solution is to fill the now-missing space from arbitrary_partition with extra space in mcuboot_secondary.
  * We will shift arbitrary partition to remain aligned with the end of flash_primary.
  * Increase the size of mcuboot_secondary by 0x1000, from 0x79000, to 0x7a000
  * This builds correctly.

* Next, we will remove the arbitrary partition, and continue increasing mcuboot secondary by 0x1000
  * This also builds as expected


* Finally, we will split the amount of space gained between mcuboot primary and secondary.
  * This also builds
  * Validating runtime behavior: as expected

* Now, we can build for xiao!
  * Unfortunately, the root cause that brought us here once again returns.
  * We are experiencing a flash_primary region overflow. 

```
c:/ncs/toolchains/c57af46cb7/opt/zephyr-sdk/arm-zephyr-eabi/bin/../lib/gcc/arm-zephyr-eabi/12.2.0/../../../../arm-zephyr-eabi/bin/ld.bfd.exe: zephyr\zephyr_pre0.elf section `rodata' will not fit in region `FLASH'
c:/ncs/toolchains/c57af46cb7/opt/zephyr-sdk/arm-zephyr-eabi/bin/../lib/gcc/arm-zephyr-eabi/12.2.0/../../../../arm-zephyr-eabi/bin/ld.bfd.exe: region `FLASH' overflowed by 2360 bytes
c:/ncs/toolchains/c57af46cb7/opt/zephyr-sdk/arm-zephyr-eabi/bin/../lib/gcc/arm-zephyr-eabi/12.2.0/../../../../arm-zephyr-eabi/bin/ld.bfd.exe: zephyr/libzephyr.a(usb_device.c.obj): in function `k_usleep':
C:\Users\finnb\Documents\auli\zephyr_experiments\mass_pm_dfu\build_xiao\mcuboot/zephyr/include/generated/syscalls/kernel.h:151: undefined reference to `z_impl_k_usleep'

```
### Attempt to recreate overflow errors without DFU

We're going to try to recreate the error by overfilling mass_pm sample's primary flash partition

* 1. Making the arbitrary flash partition small, but present.
  * Builds as expected
* 2. Making the arbitary flash partition as large as the entire primary flash region
  * Error:  
    ```Partition manager failed: Incorrect amount of gaps found in static configuration. There must be exactly one gap in the static configuration to support placing the dynamic partitions (such as 'app'). Gaps found (0): The most common solution to this problem is to fill the smallest of these gaps with statically defined partition(s) until there is only one gap left. Alternatively re-order the already defined static partitions so that only one gap remains.```
* 3. Make note of the compiled application size when arbitrary_partition is small, and pick a number such that `(app size) + (arbitrary_size) > (flash_primary_size)`
  * (First pick: 0x50 000)

```
[182/183] Linking C executable zephyr\zephyr.elf
Memory region         Used Size  Region Size  %age Used
           FLASH:       78604 B       704 KB     10.90%
             RAM:       26268 B       256 KB     10.02%
        IDT_LIST:          0 GB         2 KB      0.00%
Converting to uf2, output size: 157696, start address: 0x27000
Wrote 157696 bytes to zephyr.uf2
```
  * I observe that 157696 = 0x26 800, 
  * I believe the SOC flash size to be 1Mb = 0x100 000
  * Therefore the maximum allowable size for arbitrary partition should be believed to be 0x100 000 - 0x26 800 = 0xd9 800
  * To overflow this, I will select an arbitrary_partition_size of 0xda000.
    * The overflow here should be 200 bytes

Unfortunately, this is not the case. The project builds without complaint. I will re-determine the size of the application in Flash by using the "Used Size / Flash" metric rather than the output size.

  * The new answer is that application size is 78604 Bytes = 0x13 30c
  * The associated maximum allowed size, then is 0x10000 - 0x1330c = 0xeccf4
  * The size I will pick to overflow this is 0xecd00, which should overflow by 12 bytes

```
zephyr\zephyr_pre0.elf section `usb_cfg_data_area' will not fit in region `FLASH'
c:/ncs/toolchains/c57af46cb7/opt/zephyr-sdk/arm-zephyr-eabi/bin/../lib/gcc/arm-zephyr-eabi/12.2.0/../../../../arm-zephyr-eabi/bin/ld.bfd.exe: region `FLASH' overflowed by 12 bytes
```
  * The conclusion here is that the overflow occurs when the app partition is given insufficient size for the `Used Size / Flash` Metric.

### Identifying the size of our needed app partition for mass_pm_dfu

mcuboot complicates some of this -- instead of just one Used Size / Flash metric, there are two.

I believe one of these to be from the separately compiled bootloader (mcuboot), and the other to be the childimage / application to be booted from the bootloader.

Those two blocks are: 

```
[185/185] Linking C executable zephyr\zephyr.elf
Memory region         Used Size  Region Size  %age Used
           FLASH:       31870 B        48 KB     64.84%
             RAM:       17728 B       256 KB      6.76%
        IDT_LIST:          0 GB         2 KB      0.00%
[308/316] Linking C executable zephyr\zephyr.elf
Memory region         Used Size  Region Size  %age Used
           FLASH:      378216 B     499200 B     75.76%
             RAM:       70296 B       256 KB     26.82%
        IDT_LIST:          0 GB         2 KB      0.00%
```

To compare those values to the memory report, here is a summary

  * Flash/Used 1 = `31.87 kB = 0x7C7E`
  * Region 1 matches the MCUBoot partition size

  * Flash/Used 2 = `378216 B = 0x5C568`
  * Region 2 size matches the mcuboot_primary_app and app partition sizes (`0x79e00`)

* Memory Report

The MCUBoot partition has size `48 kB = 0xc000`

The primary and secondary mcuboot partitions each have a size of `488 kB = 0x7a000`

The mcuboot pad occupies `512 B = 0x200`

The primary app and app occupy the same region of `487.5 kB = 0x79e00`

* Conclusions

The size needed for our app (whencompiled for dev kit) is **less than 380kB.** The size of the app partition is **488 kB**. We should have 100 spare kB, which makes our overflow of 2.36 kB very confusing.

Our app should fit comfortably within this partition!

One thought that springs to mind as a potential explanation is that the lsm6dsl drivers are being linked.

The IMU is enabled in the system devicetree, possibly looping in size-increasing drivers to our compiled application.

### Disabling IMU driver linking?

The overflow amount is identical. 2.36 kB

### Located Devicetree comment -- toggle value of chosen external flash.

When `nordic,pm-ext-flash = &p25q16h;` is added to xiao_ble_sense.overlay, the overflow amount INCREASES from 2.36kB to 2.408 kB.

To be clear, the amount of overflow is LARGER when external flash is chosen by 0xa0 bytes.

## Testing to see if amount of overflow can be changed by varying prj.conf options

According to a devZone post: [https://devzone.nordicsemi.com/f/nordic-q-a/101694/nrf7002dk---region-flash-overflowed]

One can reduce application footprint using several configuration options (Sigurd Hellesvik's second response). 

We will now attempt to reduce our application footprint to change the amount of overflow

### Option 1: Use memory report

Not possible - can't build yet.

### Option 2: Optimizing Zephyr / Minimal Footprint sample

* Stack Sizes
  * These are RAM-based, so will be skipped for now
* Usused peripherals
  * We may be able to do away with some enabled-by-default peripherals
  * GPIO: overflow remains at 2408 Bytes
* Debug / Informational Options
  * Boot Banner: already disabled
  * Debug
    * disabled: 2.408 kB OverFlow
    * enabled:  2.408 kB OverFlow
* Minimal Sample
  * CONFIG_ARM_MPU_SUPPORT=n
    * Unchanged overflow size

### Option 3: Analyze stack usage.

These are still ram-based. Skipping for now

### Limit / Disable Debug / Logging.

Tried disabling debug, this is still a RAM issue.

## Continued: Testing overflow variance

### Rom report

Fails for the same reason as memory report

### Optimize for Size

2408 remains the same amount of overflow
